@@ -24,6 +24,8 @@
 #include <mozilla-config.h>
 #include <config.h>
 
+#include <string.h>
+
 #include "yelp-gecko-services.h"
 #include "yelp-gecko-utils.h"
 #include "yelp-marshal.h"
@@ -90,13 +92,20 @@ static gint
 html_open_uri (GtkMozEmbed *embed, const gchar *uri)
 {
     YelpHtml *html = YELP_HTML (embed);
-    gboolean block_load;
+    gboolean block_load = FALSE;
 
     g_return_val_if_fail (uri != NULL, FALSE);
 
     debug_print (DB_FUNCTION, "entering\n");
     debug_print (DB_ARG, "  uri = \"%s\"\n", uri);
 
+    if (g_str_equal (html->priv->base_uri, uri)) {
+	/* As of xulrunner 1.6.2, open_uri is called in response
+	   to the base URI we pass in.
+	 */
+	return FALSE;
+    }
+
     if (!html->priv->frames_enabled) {
 	g_signal_emit (html, signals[URI_SELECTED], 0, uri, FALSE);
 	block_load = TRUE;
@@ -106,9 +115,43 @@ html_open_uri (GtkMozEmbed *embed, const gchar *uri)
     return block_load;
 }
 
+#ifdef HAVE_GECKO_1_9
+static void
+html_reset_accessible_parent (GtkWidget *widget)
+{
+    AtkObject * html_acc = gtk_widget_get_accessible (widget);
+    AtkObject * parent_acc = gtk_widget_get_accessible (widget->parent);
+    if (html_acc && parent_acc) {
+	atk_object_set_parent (html_acc, parent_acc);
+    }
+}
+#endif
+
 static void
 html_realize (GtkWidget *widget)
 {
+#ifdef HAVE_GECKO_1_9
+    /* When Gecko accessibility module init, it will overwrite 
+     * atk_class->get_root.
+     * But the top level accessible of yelp is not controlled by Gecko.
+     * So we need to restore the callback. See Bug #545162.
+     * It only need to do once.
+     * We do it here because Gecko a11y module inits when it is actually used,
+     * we call gtk_widget_get_accessible to pull the trigger. */
+
+    static gboolean gail_get_root_restored = FALSE;
+    static AtkObject * (*gail_get_root) (void);
+    static AtkUtilClass * atk_class = NULL;
+    if (!gail_get_root_restored) {
+	gpointer data;
+	data = g_type_class_peek (ATK_TYPE_UTIL);
+	if (data) {
+	    atk_class = ATK_UTIL_CLASS (data);
+	    gail_get_root = atk_class->get_root;
+	}
+    }
+#endif
+
     YelpHtml *html = YELP_HTML (widget);
 
     GTK_WIDGET_CLASS (parent_class)->realize (widget);
@@ -119,6 +162,17 @@ html_realize (GtkWidget *widget)
     if (NS_FAILED (rv)) {
         g_warning ("Yelper initialization failed for %p\n", (void*) html);
     }
+
+#ifdef HAVE_GECKO_1_9
+    if (!gail_get_root_restored) {
+	gail_get_root_restored = TRUE;
+	if (atk_class && gail_get_root) {
+	    gtk_widget_get_accessible (widget);
+	    atk_class->get_root = gail_get_root;
+	}
+    }
+#endif
+  
 }
 
 static void
@@ -375,9 +429,14 @@ yelp_html_close (YelpHtml *html)
 {
     debug_print (DB_FUNCTION, "entering\n");
     gtk_moz_embed_close_stream (GTK_MOZ_EMBED (html));
-    html->priv->timeout = g_timeout_add (2000, 
-					 (GSourceFunc) timeout_update_gok,
-					 html);
+
+#ifdef HAVE_GECKO_1_9
+    html_reset_accessible_parent (GTK_WIDGET (html));
+#else
+    html->priv->timeout = g_timeout_add_seconds (2,
+						 (GSourceFunc) timeout_update_gok,
+						 html);
+#endif
 }
 
 gboolean
