@@ -504,10 +504,10 @@ toc_process (YelpToc *toc)
 
     info = gtk_icon_theme_lookup_icon (theme, "yelp-icon-big", 192, 0);
     if (info) {
-	params[params_i++] = "help_icon";
+	params[params_i++] = g_strdup ("help_icon");
 	params[params_i++] = g_strdup_printf ("\"%s\"",
 				      gtk_icon_info_get_filename (info));
-	params[params_i++] = "help_icon_size";
+	params[params_i++] = g_strdup ("help_icon_size");
 	params[params_i++] = g_strdup_printf ("%i",
 					      gtk_icon_info_get_base_size (info));
 	gtk_icon_info_free (info);
@@ -529,6 +529,7 @@ toc_process (YelpToc *toc)
     yelp_transform_start (priv->transform,
 			  priv->xmldoc,
 			  params);
+    g_strfreev (params);
     g_mutex_unlock (priv->mutex);
 
  done:
@@ -552,7 +553,7 @@ xml_trim_titles (xmlNodePtr node, xmlChar * nodetype)
 	if (!xmlStrcmp (cur->name, nodetype)) {
 	    xmlChar *cur_lang = NULL;
 	    int cur_pri = INT_MAX;
-	    cur_lang = xmlNodeGetLang (cur);
+	    cur_lang = xmlGetNsProp(cur, BAD_CAST "lang", XML_XML_NAMESPACE);
 	    if (cur_lang) {
 		for (j = 0; langs[j]; j++) {
 		    if (g_str_equal (cur_lang, langs[j])) {
